@@ -20,21 +20,26 @@ class FlashCosineSimAttention(Function):
     @staticmethod
     def forward(
         ctx,
-        q, k, v
+        q, k, v,
+        scale
     ):
-        o, l = forward(q, k, v)
+        o, l = forward(q, k, v, scale)
+
         ctx.save_for_backward(o, l, q, k, v)
-        return q
+        ctx.scale = scale
+        return o
 
     @staticmethod
     def backward(ctx, do):
         o, l, q, k, v = ctx.saved_tensors
-        dq, dk, dv = backward(do, o, l, q, k, v)
-        return dq, dk, dv
+        scale = ctx.scale
+
+        dq, dk, dv = backward(do, o, l, q, k, v, scale)
+        return dq, dk, dv, None
 
 # wrapper function
 
-def flash_cosine_sim_attention(q, k, v):
+def flash_cosine_sim_attention(q, k, v, scale = 8):
     q, k = map(l2norm, (q, k))
-    o = FlashCosineSimAttention.apply(q, k, v)
+    o = FlashCosineSimAttention.apply(q, k, v, scale)
     return o
